=== modified file 'NEWS'
--- NEWS	2006-05-11 02:23:31 +0000
+++ NEWS	2006-05-12 06:59:47 +0000
@@ -1,6 +1,14 @@
-In development:
-
-  CHANGES:
+IN DEVELOPMENT
+
+  IMPROVEMENTS:
+
+   * On Unix, detect terminal width using an ioctl not just $COLUMNS.
+     Use terminal width for single-line logs from ``bzr log --line`` and
+     pending-merge display.  (Robert Widhopf-Fenk, Gustavo Niemeyer)
+     (Malone #3507)
+
+   * On Windows, detect terminal widht using GetConsoleScreenBufferInfo.
+     (Alexander Belchenko)
   
     'bzr selftest' now shows a progress bar with the number of tests, and 
     progress made. 'make check' shows tests in -v mode, to be more useful
@@ -135,7 +143,7 @@
       and allows checkouts over SFTP to perform acceptably. 
       The initial Knit code was contributed by Johan Rydberg based on a
       specification by Martin Pool.
-      (Robert Collins, Aaron Bentley, John Rydberg, Martin Pool).
+      (Robert Collins, Aaron Bentley, Johan Rydberg, Martin Pool).
 
     * New tool to generate all-in-one html version of the manual.  (Alexander
       Belchenko)

=== modified file 'bzr'
--- bzr	2006-01-13 06:31:42 +0000
+++ bzr	2006-05-10 07:13:32 +0000
@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@
                      "\n")
     raise
 
-if bzrlib.version_info[:3] != (0, 8, 0):
+if bzrlib.version_info[:3] != (0, 9, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-05-08 03:48:39 +0000
+++ bzrlib/__init__.py	2006-05-10 07:13:32 +0000
@@ -1,4 +1,4 @@
-# (C) 2005 Canonical Development Ltd
+# Copyright (C) 2005, 2006 Canonical Development Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -89,14 +89,18 @@
 del locale
 
 __copyright__ = "Copyright 2005, 2006 Canonical Development Ltd."
-__version__ = version_string = '0.8'
+__version__ = version_string = '0.9'
 
-# same format as sys.version_info: A tuple containing the five components of
+# same format as sys.version_info: "A tuple containing the five components of
 # the version number: major, minor, micro, releaselevel, and serial. All
 # values except releaselevel are integers; the release level is 'alpha',
 # 'beta', 'candidate', or 'final'. The version_info value corresponding to the
-# Python version 2.0 is (2, 0, 0, 'final', 0).
-version_info = (0, 8, 0, 'final', 0)
+# Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
+# releaselevel of 'dev' for unreleased under-development code.
+
+version_info = (0, 9, 0, 'dev', 0)
+
+__version__ = '%d.%d.%d%s%d' % version_info
 
 
 from bzrlib.symbol_versioning import deprecated_function, zero_seven

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-05-10 17:48:25 +0000
+++ bzrlib/builtins.py	2006-05-12 06:59:47 +0000
@@ -1580,6 +1580,12 @@
         # TODO: if the commit *does* happen to fail, then save the commit 
         # message to a temporary file where it can be recovered
         tree, selected_list = tree_files(selected_list)
+        if selected_list == ['']:
+            # workaround - commit of root of tree should be exactly the same
+            # as just default commit in that tree, and succeed even though
+            # selected-file merge commit is not done yet
+            selected_list = []
+
         if local and not tree.branch.get_bound_location():
             raise errors.LocalRequiresBoundBranch()
         if message is None and not file:
@@ -2394,10 +2400,6 @@
 class cmd_uncommit(bzrlib.commands.Command):
     """Remove the last committed revision.
 
-    By supplying the --all flag, it will not only remove the entry 
-    from revision_history, but also remove all of the entries in the
-    stores.
-
     --verbose will print out what is being removed.
     --dry-run will go through all the motions, but not actually
     remove anything.

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-04-19 23:32:08 +0000
+++ bzrlib/commit.py	2006-05-11 08:17:09 +0000
@@ -296,7 +296,8 @@
 
             self._gather_parents()
             if len(self.parents) > 1 and self.specific_files:
-                raise NotImplementedError('selected-file commit of merges is not supported yet')
+                raise NotImplementedError('selected-file commit of merges is not supported yet: files %r',
+                        self.specific_files)
             self._check_parents_present()
             
             self._remove_deleted()
@@ -511,7 +512,7 @@
             if ie.revision is None:
                 change = ie.snapshot(self.rev_id, path, previous_entries,
                                      self.work_tree, self.weave_store,
-                                     self.branch.get_transaction())
+                                     self.branch.repository.get_transaction())
             else:
                 change = "unchanged"
             self.reporter.snapshot_change(change, path)

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-04-20 02:42:17 +0000
+++ bzrlib/inventory.py	2006-05-11 08:17:09 +0000
@@ -656,21 +656,21 @@
         self.text_sha1 = None
         self.executable = None
 
-    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction):
+    def _snapshot_text(self, file_parents, work_tree, versionedfile_store, transaction):
         """See InventoryEntry._snapshot_text."""
-        mutter('storing file {%s} in revision {%s}',
-               self.file_id, self.revision)
+        mutter('storing text of file {%s} in revision {%s} into %r',
+               self.file_id, self.revision, versionedfile_store)
         # special case to avoid diffing on renames or 
         # reparenting
         if (len(file_parents) == 1
             and self.text_sha1 == file_parents.values()[0].text_sha1
             and self.text_size == file_parents.values()[0].text_size):
             previous_ie = file_parents.values()[0]
-            versionedfile = weave_store.get_weave(self.file_id, transaction)
+            versionedfile = versionedfile_store.get_weave(self.file_id, transaction)
             versionedfile.clone_text(self.revision, previous_ie.revision, file_parents.keys())
         else:
             new_lines = work_tree.get_file(self.file_id).readlines()
-            self._add_text_to_weave(new_lines, file_parents.keys(), weave_store,
+            self._add_text_to_weave(new_lines, file_parents.keys(), versionedfile_store,
                                     transaction)
             self.text_sha1 = sha_strings(new_lines)
             self.text_size = sum(map(len, new_lines))

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-05-03 10:01:43 +0000
+++ bzrlib/knit.py	2006-05-10 08:26:30 +0000
@@ -290,6 +290,10 @@
         self._data = _KnitData(transport, relpath + DATA_SUFFIX,
             access_mode, create=create and not len(self), file_mode=file_mode)
 
+    def __repr__(self):
+        return '%s(%s)' % (self.__class__.__name__, 
+                           self.transport.abspath(self.filename))
+    
     def _add_delta(self, version_id, parents, delta_parent, sha1, noeol, delta):
         """See VersionedFile._add_delta()."""
         self._check_add(version_id, []) # should we check the lines ?

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2006-03-28 14:29:13 +0000
+++ bzrlib/log.py	2006-05-09 07:09:48 +0000
@@ -443,7 +443,8 @@
             return rev.message
 
     def show(self, revno, rev, delta):
-        print >> self.to_file, self.log_string(rev, 79) 
+        from bzrlib.osutils import terminal_width
+        print >> self.to_file, self.log_string(rev, terminal_width() - 1) 
 
     def log_string(self, rev, max_chars):
         out = [self.truncate(self.short_committer(rev), 20)]

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-05-05 01:15:53 +0000
+++ bzrlib/osutils.py	2006-05-09 07:16:38 +0000
@@ -39,6 +39,7 @@
                            IllegalPath,
                            )
 from bzrlib.trace import mutter
+import bzrlib.win32console
 
 
 def make_readonly(filename):
@@ -667,16 +668,26 @@
 
 def terminal_width():
     """Return estimated terminal width."""
-
-    # TODO: Do something smart on Windows?
-
-    # TODO: Is there anything that gets a better update when the window
-    # is resized while the program is running? We could use the Python termcap
-    # library.
+    if sys.platform == 'win32':
+        import bzrlib.win32console
+        return bzrlib.win32console.get_console_size()[0]
+    width = 0
     try:
-        return int(os.environ['COLUMNS'])
-    except (IndexError, KeyError, ValueError):
-        return 80
+        import struct, fcntl, termios
+        s = struct.pack('HHHH', 0, 0, 0, 0)
+        x = fcntl.ioctl(1, termios.TIOCGWINSZ, s)
+        width = struct.unpack('HHHH', x)[1]
+    except IOError:
+        pass
+    if width <= 0:
+        try:
+            width = int(os.environ['COLUMNS'])
+        except:
+            pass
+    if width <= 0:
+        width = 80
+
+    return width
 
 def supports_executable():
     return sys.platform != "win32"

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-05-05 08:29:29 +0000
+++ bzrlib/repository.py	2006-05-11 08:17:09 +0000
@@ -167,6 +167,10 @@
         # TODO: make sure to construct the right store classes, etc, depending
         # on whether escaping is required.
 
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, 
+                           self.bzrdir.transport.base)
+
     def is_locked(self):
         return self.control_files.is_locked()
 

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2006-04-03 04:20:11 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2006-05-11 08:17:09 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2006-04-19 05:11:59 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2006-05-11 08:17:09 +0000
@@ -37,6 +37,27 @@
         self.build_tree(['hello.txt'])
         self.runbzr("commit -m empty", retcode=3)
 
+    def test_commit_with_path(self):
+        """Commit tree with path of root specified"""
+        self.run_bzr('init', 'a')
+        self.build_tree(['a/a_file'])
+        self.run_bzr('add', 'a/a_file')
+        self.run_bzr('commit', '-m', 'first commit', 'a')
+
+        self.run_bzr('branch', 'a', 'b')
+        self.build_tree_contents([('b/a_file', 'changes in b')])
+        self.run_bzr('commit', '-m', 'first commit in b', 'b')
+
+        self.build_tree_contents([('a/a_file', 'new contents')])
+        self.run_bzr('commit', '-m', 'change in a', 'a')
+
+        os.chdir('b')
+        self.run_bzr('merge', '../a', retcode=1) # will conflict
+        os.chdir('..')
+        self.run_bzr('resolved', 'b/a_file')
+        self.run_bzr('commit', '-m', 'merge into b', 'b')
+
+
     def test_10_verbose_commit(self):
         """Add one file and examine verbose commit output"""
         self.runbzr("init")
@@ -165,3 +186,35 @@
         self.assertEqualDiff('', out)
         self.assertEqualDiff('bzr: ERROR: Cannot perform local-only commits '
                              'on unbound branches.\n', err)
+
+    def test_commit_a_text_merge_in_a_checkout(self):
+        # checkouts perform multiple actions in a transaction across bond
+        # branches and their master, and have been observed to fail in the
+        # past. This is a user story reported to fail in bug #43959 where 
+        # a merge done in a checkout (using the update command) failed to
+        # commit correctly.
+        self.run_bzr('init', 'trunk')
+
+        self.run_bzr('checkout', 'trunk', 'u1')
+        self.build_tree_contents([('u1/hosts', 'initial contents')])
+        self.run_bzr('add', 'u1/hosts')
+        self.run_bzr('commit', '-m', 'add hosts', 'u1')
+
+        self.run_bzr('checkout', 'trunk', 'u2')
+        self.build_tree_contents([('u2/hosts', 'altered in u2')])
+        self.run_bzr('commit', '-m', 'checkin from u2', 'u2')
+
+        # make an offline commits
+        self.build_tree_contents([('u1/hosts', 'first offline change in u1')])
+        self.run_bzr('commit', '-m', 'checkin offline', '--local', 'u1')
+
+        # now try to pull in online work from u2, and then commit our offline
+        # work as a merge
+        # retcode 1 as we expect a text conflict
+        self.run_bzr('update', 'u1', retcode=1)
+        self.run_bzr('resolved', 'u1/hosts')
+        # add a text change here to represent resolving the merge conflicts in
+        # favour of a new version of the file not identical to either the u1
+        # version or the u2 version.
+        self.build_tree_contents([('u1/hosts', 'merge resolution\n')])
+        self.run_bzr('commit', '-m', 'checkin merge of the offline work from u1', 'u1')

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2006-05-05 01:29:34 +0000
+++ bzrlib/tests/blackbox/test_info.py	2006-05-09 10:29:44 +0000
@@ -129,11 +129,8 @@
 
         # Branch and bind to standalone, needs upgrade to metadir
         # (creates backup as unknown)
-        # XXX: I can't get this to work through API
-        self.runbzr('branch standalone bound')
-        #branch3 = branch1.bzrdir.sprout('bound').open_branch()
-        self.runbzr('upgrade --format=metaweave bound')
-        #bzrlib.upgrade.upgrade('bound', bzrlib.bzrdir.BzrDirMetaFormat1())
+        branch1.bzrdir.sprout('bound')
+        bzrlib.upgrade.upgrade('bound', bzrlib.bzrdir.BzrDirMetaFormat1())
         branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
         branch3.bind(branch1)
         out, err = self.runbzr('info bound')

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-04-13 03:12:21 +0000
+++ bzrlib/tests/test_commit.py	2006-05-11 08:17:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -392,3 +392,33 @@
             self.assertRaises(LockContention, wt.commit, 'silly')
         finally:
             master_branch.unlock()
+
+    def test_commit_bound_merge(self):
+        # see bug #43959; commit of a merge in a bound branch fails to push
+        # the new commit into the master
+        master_branch = self.make_branch('master')
+        bound_tree = self.make_branch_and_tree('bound')
+        bound_tree.branch.bind(master_branch)
+
+        self.build_tree_contents([('bound/content_file', 'initial contents\n')])
+        bound_tree.add(['content_file'])
+        bound_tree.commit(message='woo!')
+
+        other_bzrdir = master_branch.bzrdir.sprout('other')
+        other_tree = other_bzrdir.open_workingtree()
+
+        # do a commit to the the other branch changing the content file so
+        # that our commit after merging will have a merged revision in the
+        # content file history.
+        self.build_tree_contents([('other/content_file', 'change in other\n')])
+        other_tree.commit('change in other')
+
+        # do a merge into the bound branch from other, and then change the
+        # content file locally to force a new revision (rather than using the
+        # revision from other). This forces extra processing in commit.
+        self.merge(other_tree.branch, bound_tree)
+        self.build_tree_contents([('bound/content_file', 'change in bound\n')])
+
+        # before #34959 was fixed, this failed with 'revision not present in
+        # weave' when trying to implicitly push from the bound branch to the master
+        bound_tree.commit(message='commit of merge in bound tree')

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2006-05-04 10:36:13 +0000
+++ bzrlib/tests/test_lockdir.py	2006-05-08 04:39:50 +0000
@@ -162,7 +162,10 @@
             self.assertRaises(LockContention, lf2.wait_lock,
                               timeout=0.4, poll=0.1)
             after = time.time()
-            self.assertTrue(after - before <= 1.0)
+            # it should only take about 0.4 seconds, but we allow more time in
+            # case the machine is heavily loaded
+            self.assertTrue(after - before <= 8.0, 
+                    "took %f seconds to detect lock contention" % (after - before))
         finally:
             lf1.unlock()
 

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2006-05-09 03:17:13 +0000
+++ bzrlib/tests/test_ui.py	2006-05-12 06:59:47 +0000
@@ -19,6 +19,7 @@
 
 import os
 from StringIO import StringIO
+import re
 import sys
 
 import bzrlib
@@ -163,7 +164,13 @@
             self.apply_redirected(
                 None, factory.stdout, factory.stdout, factory.get_boolean, "what do you want")
             )
-        self.assertEqual(
-            "\r/ [                                                                   ] foo 0/1"
-            "\r                                                                               "
-            "\rwhat do you want? [y/n]:what do you want? [y/n]:", factory.stdout.getvalue())
+        # use a regular expression so that we don't depend on the particular
+        # screen width - could also set and restore $COLUMN if that has
+        # priority on all platforms, but it doesn't at present.
+        output = factory.stdout.getvalue()
+        if not re.match(
+            "\r/ \\[    *\\] foo 0/1"
+            "\r   *" 
+            "\rwhat do you want\\? \\[y/n\\]:what do you want\\? \\[y/n\\]:", 
+            output):
+            self.fail("didn't match factory output %r" % factory)

=== modified file 'bzrlib/win32console.py'
--- bzrlib/win32console.py	2005-10-21 03:33:18 +0000
+++ bzrlib/win32console.py	2006-05-09 07:16:38 +0000
@@ -13,6 +13,11 @@
    ctypes = None
 
 
+WIN32_STDIN_HANDLE = -10
+WIN32_STDOUT_HANDLE = -11
+WIN32_STDERR_HANDLE = -12
+
+
 def get_console_size(defaultx=80, defaulty=25):
    """ Return size of current console.
 
@@ -26,7 +31,9 @@
        # no ctypes is found
        return (defaultx, defaulty)
 
-   h = ctypes.windll.kernel32.GetStdHandle(-11)
+   # To avoid problem with redirecting output via pipe
+   # need to use stderr instead of stdout
+   h = ctypes.windll.kernel32.GetStdHandle(WIN32_STDERR_HANDLE)
    csbi = ctypes.create_string_buffer(22)
    res = ctypes.windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
 

=== removed directory 'notes'
